Option Strict On

Imports System
Imports System.Text
Imports System.Data
Imports System.IO
Imports System.Collections
Imports System.Web
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports System.Web.UI.Design
Imports Microsoft.VisualBasic   ' needed for IIf definition
Imports Microsoft.VisualBasic.DateAndTime
Imports System.ComponentModel
Imports System.ComponentModel.Design
Imports System.Reflection    ' PropertyInfo definition

' -------------------------- Databound Schedule Controls v1.6/1.9 -----------------------
'
' ------------------------------ www.rekenwonder.com ---------------------------------

Namespace rw
    ''' -----------------------------------------------------------------------------
    ''' Project	 : schedule
    ''' Class	 : rw.ScheduleCalendarDesigner
    '''
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' ScheduleCalendarDesigner adds design-time support for the ScheduleCalendar control
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    Public Class ScheduleCalendarDesigner
        Inherits ScheduleDesigner

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Provides a dummy DataTable object, allowing the ScheduleCalendar control to
        ''' show some sample data in the designer.
        ''' </summary>
        ''' <param name="minimumRows"></param>
        ''' <returns>A dummy DataTable object</returns>
        ''' -----------------------------------------------------------------------------
        Protected Overrides Function GetDummyDataTable(ByVal minimumRows As Integer) As DataTable
            'Create a new dummy table every time this function gets called.
            'The alternative is to raise OnDataSourceChanged every time DateField, TimeFieldsContainDate etc. change
            Dim control As ScheduleCalendar = CType(Me.Component, ScheduleCalendar)
            Dim dummyDataTable As DataTable
            dummyDataTable = New DataTable
            dummyDataTable.Columns.Add(New DataColumn(control.StartTimeField, System.Type.GetType("System.DateTime")))
            If (control.EndTimeField <> control.StartTimeField) Then
                dummyDataTable.Columns.Add(New DataColumn(control.EndTimeField, System.Type.GetType("System.DateTime")))
            End If
            If (Not control.TimeFieldsContainDate) Then
                If (control.DateField Is Nothing Or control.DateField = "") Then
                    Throw New HttpException("When TimeFieldsContainDate=False, DateField may not be blank")
                End If
                dummyDataTable.Columns.Add(New DataColumn(control.DateField, System.Type.GetType("System.DateTime")))
            End If
            If (control.ItemStyleField <> "") Then
                dummyDataTable.Columns.Add(New DataColumn(control.ItemStyleField, System.Type.GetType("System.String")))
            Else
                dummyDataTable.Columns.Add(New DataColumn("DummyStyleField", System.Type.GetType("System.String")))
            End If

            ' create some dummy rows
            control.ShiftStartDate()
            Dim d As Date = control.StartDate()
            Dim i As Integer
            For i = 0 To minimumRows - 1
                Dim row As DataRow = dummyDataTable.NewRow()
                d = d.AddDays(1)   ' create rows on successive days
                If (Not control.TimeFieldsContainDate) Then
                    ' put the date in the DateField, and the times in the TimeFields
                    row.Item(0) = New DateTime(1, 1, 1, 9 + i Mod 2, 0, 0)
                    If (control.EndTimeField <> control.StartTimeField) Then
                        row.Item(1) = New DateTime(1, 1, 1, 12 + i Mod 2, 0, 0)
                        row.Item(2) = d
                        row.Item(3) = "dummystyle"
                    Else
                        row.Item(1) = d
                        row.Item(2) = "dummystyle"
                    End If
                Else
                    ' ignore the DateField, and put the dates AND times in the TimeFields
                    row.Item(0) = New DateTime(d.Year, d.Month, d.Day, 9 + i Mod 2, 0, 0)
                    If (control.EndTimeField <> control.StartTimeField) Then
                        row.Item(1) = New DateTime(d.Year, d.Month, d.Day, 12 + i Mod 2, 0, 0)
                    End If
                End If
                dummyDataTable.Rows.Add(row)
            Next
            Return dummyDataTable
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Shows a Property Builder dialog box
        ''' </summary>
        ''' <param name="sender"></param>
        ''' <param name="e"></param>
        ''' -----------------------------------------------------------------------------
        Public Overrides Sub OnPropertyBuilder(ByVal sender As Object, ByVal e As EventArgs)
            Dim control As ScheduleCalendar = CType(Me.Component, ScheduleCalendar)
            Dim pd As New CalendarPropertyBuilder
            pd.tbDateField.Enabled = Not control.TimeFieldsContainDate
            pd.tbDateField.Text = control.DateField
            pd.tbStartTimeField.Text = control.StartTimeField
            pd.tbEndTimeField.Text = control.EndTimeField
            pd.tbItemStyleField.Text = control.ItemStyleField
            pd.tbDateFormatString.Text = control.DateFormatString
            pd.tbTimeFormatString.Text = control.TimeFormatString
            pd.cbTimeFieldsContainDate.Checked = control.TimeFieldsContainDate
            pd.cbIncludeEndValue.Checked = control.IncludeEndValue
            pd.cbShowValueMarks.Checked = control.ShowValueMarks
            pd.tbNumberOfDays.Text = CStr(control.NumberOfDays)
            pd.tbNumberOfRepetitions.Text = CStr(control.NumberOfRepetitions)
            pd.cbStartDay.SelectedIndex = CInt(control.StartDay)
            pd.rbHorizontal.Checked = (control.Layout = LayoutEnum.Horizontal)
            pd.rbVertical.Checked = (control.Layout = LayoutEnum.Vertical)
            pd.cbFullTimeScale.Checked = control.FullTimeScale
            pd.tbStartTimeScale.Text = control.StartOfTimeScale.ToString()
            pd.tbEndTimeScale.Text = control.EndOfTimeScale.ToString()
            pd.tbInterval.Text = control.TimeScaleInterval.ToString()
            pd.tbStartTimeScale.Enabled = control.FullTimeScale
            pd.tbEndTimeScale.Enabled = control.FullTimeScale
            pd.tbInterval.Enabled = control.FullTimeScale
            If (pd.ShowDialog() = Windows.Forms.DialogResult.OK) Then
                control.DateField = pd.tbDateField.Text
                control.StartTimeField = pd.tbStartTimeField.Text
                control.EndTimeField = pd.tbEndTimeField.Text
                control.ItemStyleField = pd.tbItemStyleField.Text
                control.DateFormatString = pd.tbDateFormatString.Text
                control.TimeFormatString = pd.tbTimeFormatString.Text
                control.TimeFieldsContainDate = pd.cbTimeFieldsContainDate.Checked
                control.IncludeEndValue = pd.cbIncludeEndValue.Checked
                control.ShowValueMarks = pd.cbShowValueMarks.Checked
                control.NumberOfDays = CInt(pd.tbNumberOfDays.Text)
                control.NumberOfRepetitions = CInt(pd.tbNumberOfRepetitions.Text)
                control.StartDay = CType(pd.cbStartDay.SelectedIndex, DayOfWeek)
                control.Layout = CType(IIf(pd.rbHorizontal.Checked, LayoutEnum.Horizontal, LayoutEnum.Vertical), LayoutEnum)
                control.FullTimeScale = pd.cbFullTimeScale.Checked
                control.StartOfTimeScale = TimeSpan.Parse(pd.tbStartTimeScale.Text)
                control.EndOfTimeScale = TimeSpan.Parse(pd.tbEndTimeScale.Text)
                control.TimeScaleInterval = Int32.Parse(pd.tbInterval.Text)
                RaiseComponentChanged(Nothing, Nothing, Nothing)
                UpdateDesignTimeHtml()
            End If
        End Sub

    End Class

End Namespace

